@@ -17,9 +17,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <assert.h>
 #include <limits.h>
@@ -46,6 +44,69 @@
 
 #define WGS84_DATUM_STRING "EPSG:4326"
 
+int fix_station(prefix *fix_name, double* coords) {
+    fix_name->sflags |= BIT(SFLAGS_FIXED);
+    node *stn = StnFromPfx(fix_name);
+    if (fixed(stn)) {
+	if (coords[0] != POS(stn, 0) ||
+	    coords[1] != POS(stn, 1) ||
+	    coords[2] != POS(stn, 2)) {
+	    return -1;
+	}
+	return 1;
+    }
+
+    POS(stn, 0) = coords[0];
+    POS(stn, 1) = coords[1];
+    POS(stn, 2) = coords[2];
+    fix(stn);
+
+    // Make the station's file:line location reflect where it was fixed.
+    fix_name->filename = file.filename;
+    fix_name->line = file.line;
+    return 0;
+}
+
+void fix_station_with_variance(prefix *fix_name, double* coords,
+			       real var_x, real var_y, real var_z,
+#ifndef NO_COVARIANCES
+			       real cxy, real cyz, real czx
+#endif
+			      )
+{
+    node *stn = StnFromPfx(fix_name);
+    if (!fixed(stn)) {
+	node *fixpt = osnew(node);
+	prefix *name;
+	name = osnew(prefix);
+	name->pos = osnew(pos);
+	name->ident = NULL;
+	name->shape = 0;
+	fixpt->name = name;
+	name->stn = fixpt;
+	name->up = NULL;
+	if (TSTBIT(pcs->infer, INFER_EXPORTS)) {
+	    name->min_export = USHRT_MAX;
+	} else {
+	    name->min_export = 0;
+	}
+	name->max_export = 0;
+	name->sflags = 0;
+	add_stn_to_list(&stnlist, fixpt);
+	POS(fixpt, 0) = coords[0];
+	POS(fixpt, 1) = coords[1];
+	POS(fixpt, 2) = coords[2];
+	fix(fixpt);
+	fixpt->leg[0] = fixpt->leg[1] = fixpt->leg[2] = NULL;
+	addfakeleg(fixpt, stn, 0, 0, 0,
+		   var_x, var_y, var_z
+#ifndef NO_COVARIANCES
+		   , cxy, cyz, czx
+#endif
+		  );
+    }
+}
+
 static void
 default_grade(settings *s)
 {
@@ -249,7 +310,7 @@ default_all(settings *s)
 
 string token = S_INIT;
 
-static string uctoken = S_INIT;
+string uctoken = S_INIT;
 
 /* read token */
 extern void
@@ -275,10 +336,23 @@ get_token_no_blanks(void)
 #endif
 }
 
+extern void
+get_token_walls(void)
+{
+   skipblanks();
+   s_clear(&token);
+   s_clear(&uctoken);
+   while (isalnum(ch)) {
+      s_catchar(&token, ch);
+      s_catchar(&uctoken, toupper(ch));
+      nextch();
+   }
+}
+
 static string word = S_INIT;
 
 /* read word */
-static void
+void
 get_word(void)
 {
    s_clear(&word);
@@ -580,7 +654,7 @@ cmd_set(void)
 	 if (++root_depr_count == 5)
 	     /* TRANSLATORS: If you're unsure what "deprecated" means, see:
 	      * https://en.wikipedia.org/wiki/Deprecation */
-	    compile_diagnostic(DIAG_WARN, /*Further uses of this deprecated feature will not be reported*/95);
+	    compile_diagnostic(DIAG_INFO, /*Further uses of this deprecated feature will not be reported*/95);
       }
    }
 #endif
@@ -665,8 +739,8 @@ check_reentry(prefix *survey, const filepos* fpos_ptr)
        *
        * Would lead to:
        *
-       * crawl.svx:4:8: Reentering an existing survey is deprecated
-       * crawl.svx:1: Originally entered here
+       * crawl.svx:4:8: warning: Reentering an existing survey is deprecated
+       * crawl.svx:1: info: Originally entered here
        *
        * If you're unsure what "deprecated" means, see:
        * https://en.wikipedia.org/wiki/Deprecation */
@@ -684,19 +758,19 @@ check_reentry(prefix *survey, const filepos* fpos_ptr)
        *
        * Would lead to:
        *
-       * crawl.svx:4:8: Reentering an existing survey is deprecated
-       * crawl.svx:1: Originally entered here
+       * crawl.svx:4:8: warning: Reentering an existing survey is deprecated
+       * crawl.svx:1: info: Originally entered here
        *
        * If you're unsure what "deprecated" means, see:
        * https://en.wikipedia.org/wiki/Deprecation */
-      compile_diagnostic_pfx(DIAG_WARN, survey, /*Originally entered here*/30);
+      compile_diagnostic_pfx(DIAG_INFO, survey, /*Originally entered here*/30);
       if (++reenter_depr_count == 5) {
 	 /* After we've warned about 5 uses of the same deprecated feature, we
 	  * give up for the rest of the current processing run.
 	  *
 	  * If you're unsure what "deprecated" means, see:
 	  * https://en.wikipedia.org/wiki/Deprecation */
-	 compile_diagnostic(DIAG_WARN, /*Further uses of this deprecated feature will not be reported*/95);
+	 compile_diagnostic(DIAG_INFO, /*Further uses of this deprecated feature will not be reported*/95);
       }
    } else {
       survey->sflags |= BIT(SFLAGS_PREFIX_ENTERED);
@@ -720,7 +794,7 @@ cmd_prefix(void)
        * https://en.wikipedia.org/wiki/Deprecation */
       compile_diagnostic(DIAG_WARN|DIAG_TOKEN, /**prefix is deprecated - use *begin and *end instead*/6);
       if (++prefix_depr_count == 5)
-	 compile_diagnostic(DIAG_WARN, /*Further uses of this deprecated feature will not be reported*/95);
+	 compile_diagnostic(DIAG_INFO, /*Further uses of this deprecated feature will not be reported*/95);
    }
    get_pos(&fp);
    survey = read_prefix(PFX_SURVEY|PFX_ALLOW_ROOT);
@@ -824,7 +898,8 @@ report_declination(settings *p)
 	PUTC(' ', STDERR);
 	fputs(p->dec_context, STDERR);
 	fputnl(STDERR);
-	free(p->dec_context);
+	if (p->next && p->dec_context != p->next->dec_context)
+	    free(p->dec_context);
 	p->dec_context = NULL;
 	p->min_declination = HUGE_VAL;
 	p->max_declination = -HUGE_VAL;
@@ -941,7 +1016,8 @@ cmd_end(void)
    if (pcs->begin_lineno == 0) {
       if (pcs->next == NULL) {
 	 /* more ENDs than BEGINs */
-	 compile_diagnostic(DIAG_ERR|DIAG_SKIP, /*No matching BEGIN*/192);
+	 /* TRANSLATORS: %s is replaced with e.g. BEGIN or .BOOK or #[ */
+	 compile_diagnostic(DIAG_ERR|DIAG_SKIP, /*No matching %s*/192, "BEGIN");
       } else {
 	 compile_diagnostic(DIAG_ERR|DIAG_SKIP, /*END with no matching BEGIN in this file*/22);
       }
@@ -994,7 +1070,7 @@ static void
 cmd_entrance(void)
 {
    prefix *pfx = read_prefix(PFX_STATION);
-   pfx->sflags |= BIT(SFLAGS_ENTRANCE);
+   pfx->sflags |= BIT(SFLAGS_ENTRANCE) | BIT(SFLAGS_USED);
 }
 
 static const prefix * first_fix_name = NULL;
@@ -1004,17 +1080,14 @@ static unsigned first_fix_line;
 static void
 cmd_fix(void)
 {
-   prefix *fix_name;
-   node *stn = NULL;
    static prefix *name_omit_already = NULL;
    static const char * name_omit_already_filename = NULL;
    static unsigned int name_omit_already_line;
-   real x, y, z;
+   PJ_COORD coord;
    filepos fp_stn, fp;
 
    get_pos(&fp_stn);
-   fix_name = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
-   fix_name->sflags |= BIT(SFLAGS_FIXED);
+   prefix *fix_name = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
 
    get_pos(&fp);
    get_token();
@@ -1025,13 +1098,13 @@ cmd_fix(void)
       if (!s_empty(&uctoken)) set_pos(&fp);
    }
 
-   x = read_numeric(true);
-   if (x == HUGE_REAL) {
+   coord.v[0] = read_numeric(true);
+   if (coord.v[0] == HUGE_REAL) {
       /* If the end of the line isn't blank, read a number after all to
        * get a more helpful error message */
-      if (!isEol(ch) && !isComm(ch)) x = read_numeric(false);
+      if (!isEol(ch) && !isComm(ch)) coord.v[0] = read_numeric(false);
    }
-   if (x == HUGE_REAL) {
+   if (coord.v[0] == HUGE_REAL) {
       if (pcs->proj_str || proj_str_out) {
 	 compile_diagnostic(DIAG_ERR|DIAG_COL|DIAG_SKIP, /*Coordinates can't be omitted when coordinate system has been specified*/439);
 	 return;
@@ -1060,11 +1133,10 @@ cmd_fix(void)
 	 name_omit_already_line = file.line;
       }
 
-      x = y = z = (real)0.0;
+      coord.v[0] = coord.v[1] = coord.v[2] = (real)0.0;
    } else {
-      real sdx;
-      y = read_numeric(false);
-      z = read_numeric(false);
+      coord.v[1] = read_numeric(false);
+      coord.v[2] = read_numeric(false);
 
       if (pcs->proj_str && proj_str_out) {
 	 PJ *transform = pj_cached;
@@ -1089,22 +1161,21 @@ cmd_fix(void)
 
 	 if (proj_angular_input(transform, PJ_FWD)) {
 	    /* Input coordinate system expects radians. */
-	    x = rad(x);
-	    y = rad(y);
+	    coord.v[0] = rad(coord.v[0]);
+	    coord.v[1] = rad(coord.v[1]);
 	 }
 
-	 PJ_COORD coord = {{x, y, z, HUGE_VAL}};
+	 coord.v[3] = HUGE_VAL;
 	 coord = proj_trans(transform, PJ_FWD, coord);
-	 x = coord.xyzt.x;
-	 y = coord.xyzt.y;
-	 z = coord.xyzt.z;
 
-	 if (x == HUGE_VAL || y == HUGE_VAL || z == HUGE_VAL) {
+	 if (coord.v[0] == HUGE_VAL ||
+	     coord.v[1] == HUGE_VAL ||
+	     coord.v[2] == HUGE_VAL) {
 	    compile_diagnostic(DIAG_ERR, /*Failed to convert coordinates: %s*/436,
 			       proj_context_errno_string(PJ_DEFAULT_CTX,
 							 proj_errno(transform)));
 	    /* Set dummy values which are finite. */
-	    x = y = z = 0;
+	    coord.v[0] = coord.v[1] = coord.v[2] = 0;
 	 }
       } else if (pcs->proj_str) {
 	 compile_diagnostic(DIAG_ERR, /*The input projection is set but the output projection isn't*/437);
@@ -1113,7 +1184,7 @@ cmd_fix(void)
       }
 
       get_pos(&fp);
-      sdx = read_numeric(true);
+      real sdx = read_numeric(true);
       if (sdx <= 0) {
 	  set_pos(&fp);
 	  compile_diagnostic(DIAG_ERR|DIAG_SKIP|DIAG_NUM, /*Standard deviation must be positive*/48);
@@ -1155,37 +1226,12 @@ cmd_fix(void)
 	       }
 	    }
 	 }
-	 stn = StnFromPfx(fix_name);
-	 if (!fixed(stn)) {
-	    node *fixpt = osnew(node);
-	    prefix *name;
-	    name = osnew(prefix);
-	    name->pos = osnew(pos);
-	    name->ident = NULL;
-	    name->shape = 0;
-	    fixpt->name = name;
-	    name->stn = fixpt;
-	    name->up = NULL;
-	    if (TSTBIT(pcs->infer, INFER_EXPORTS)) {
-	       name->min_export = USHRT_MAX;
-	    } else {
-	       name->min_export = 0;
-	    }
-	    name->max_export = 0;
-	    name->sflags = 0;
-	    add_stn_to_list(&stnlist, fixpt);
-	    POS(fixpt, 0) = x;
-	    POS(fixpt, 1) = y;
-	    POS(fixpt, 2) = z;
-	    fix(fixpt);
-	    fixpt->leg[0] = fixpt->leg[1] = fixpt->leg[2] = NULL;
-	    addfakeleg(fixpt, stn, 0, 0, 0,
-		       sdx * sdx, sdy * sdy, sdz * sdz
+	 fix_station_with_variance(fix_name, coord.v,
+				   sdx * sdx, sdy * sdy, sdz * sdz
 #ifndef NO_COVARIANCES
-		       , cxy, cyz, czx
+				   , cxy, cyz, czx
 #endif
-		       );
-	 }
+				  );
 
 	 if (!first_fix_name) {
 	    /* We track if we've fixed a station yet, and if so what the name
@@ -1209,23 +1255,20 @@ cmd_fix(void)
       first_fix_line = file.line;
    }
 
-   stn = StnFromPfx(fix_name);
-   if (!fixed(stn)) {
-      POS(stn, 0) = x;
-      POS(stn, 1) = y;
-      POS(stn, 2) = z;
-      fix(stn);
+   int fix_result = fix_station(fix_name, coord.v);
+   if (fix_result == 0) {
       return;
    }
 
    get_pos(&fp);
    set_pos(&fp_stn);
-   if (x != POS(stn, 0) || y != POS(stn, 1) || z != POS(stn, 2)) {
+   if (fix_result < 0) {
        compile_diagnostic(DIAG_ERR|DIAG_WORD, /*Station already fixed or equated to a fixed point*/46);
    } else {
        /* TRANSLATORS: *fix a 1 2 3 / *fix a 1 2 3 */
        compile_diagnostic(DIAG_WARN|DIAG_WORD, /*Station already fixed at the same coordinates*/55);
    }
+   compile_diagnostic_pfx(DIAG_INFO, fix_name, /*Previously fixed or equated here*/493);
    set_pos(&fp);
 }
 
@@ -1267,26 +1310,34 @@ cmd_flags(void)
    }
 
    if (fNot) {
-      compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Expecting “DUPLICATE”, “SPLAY”, or “SURFACE”*/188);
+      compile_diagnostic(DIAG_ERR|DIAG_TOKEN,
+			 /*Expecting “%s”, “%s”, or “%s”*/188,
+			 "DUPLICATE", "SPLAY", "SURFACE");
    } else if (fEmpty) {
-      compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Expecting “NOT”, “DUPLICATE”, “SPLAY”, or “SURFACE”*/189);
+      compile_diagnostic(DIAG_ERR|DIAG_TOKEN,
+			 /*Expecting “%s”, “%s”, “%s”, or “%s”*/189,
+			 "NOT", "DUPLICATE", "SPLAY", "SURFACE");
    }
 }
 
 static void
 cmd_equate(void)
 {
-   prefix *name1, *name2;
-   bool fOnlyOneStn = true; /* to trap eg *equate entrance.6 */
    filepos fp;
-
    get_pos(&fp);
-   name1 = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO);
+   prefix *prev_name = NULL;
+   prefix *name = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO);
    while (true) {
-      name2 = name1;
+      if (!name->stn || !fixed(name->stn)) {
+	  // If the station isn't already fixed, make its file:line location
+	  // reflect this *equate.
+	  name->filename = file.filename;
+	  name->line = file.line;
+      }
       skipblanks();
       if (isEol(ch) || isComm(ch)) {
-	 if (fOnlyOneStn) {
+	 if (prev_name == NULL) {
+	    /* E.g. *equate entrance.6 */
 	    set_pos(&fp);
 	    /* TRANSLATORS: EQUATE is a command name, so shouldn’t be
 	     * translated.
@@ -1298,9 +1349,9 @@ cmd_equate(void)
 	 return;
       }
 
-      name1 = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO);
-      process_equate(name1, name2);
-      fOnlyOneStn = false;
+      prev_name = name;
+      name = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO);
+      process_equate(name, prev_name);
    }
 }
 
@@ -1349,11 +1400,8 @@ cmd_export(void)
    fExportUsed = true;
    do {
       int depth = 0;
-      pfx = read_prefix(PFX_STATION|PFX_NEW);
-      if (pfx == NULL) {
-	 /* The argument was an existing station. */
-	 /* FIXME */
-      } else {
+      pfx = read_prefix(PFX_STATION);
+      {
 	 prefix *p = pfx;
 	 while (p != NULL && p != pcs->Prefix) {
 	    depth++;
@@ -1538,7 +1586,7 @@ cmd_data(void)
 	 compile_diagnostic(DIAG_WARN|DIAG_TOKEN, /*“*data %s %c …” is deprecated - use “*data %s …” instead*/104,
 			    s_str(&token), ch, s_str(&token));
 	 if (++data_depr_count == 5)
-	    compile_diagnostic(DIAG_WARN, /*Further uses of this deprecated feature will not be reported*/95);
+	    compile_diagnostic(DIAG_INFO, /*Further uses of this deprecated feature will not be reported*/95);
       }
       nextch();
    }
@@ -1958,7 +2006,7 @@ cmd_default(void)
        * https://en.wikipedia.org/wiki/Deprecation */
       compile_diagnostic(DIAG_WARN|DIAG_COL, /**DEFAULT is deprecated - use *CALIBRATE/DATA/SD/UNITS with argument DEFAULT instead*/20);
       if (++default_depr_count == 5)
-	 compile_diagnostic(DIAG_WARN, /*Further uses of this deprecated feature will not be reported*/95);
+	 compile_diagnostic(DIAG_INFO, /*Further uses of this deprecated feature will not be reported*/95);
    }
 
    get_token();
