@@ -20,9 +20,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include "export.h"
 
@@ -35,6 +33,7 @@
 #include "json.h"
 #include "kml.h"
 #include "mainfrm.h"
+#include "osalloc.h"
 #include "pos.h"
 
 #include <float.h>
