@@ -27,18 +27,21 @@
 
 class Export3D : public ExportFilter {
   private:
+    wxString cs;
     char separator;
 
     img* pimg = nullptr;
 
   public:
-    Export3D(char separator_) : separator(separator_) { }
+    Export3D(const wxString cs_, char separator_)
+	: cs(cs_), separator(separator_) { }
+
     ~Export3D();
     const int * passes() const;
     void header(const char *, const char *, time_t,
 		double, double, double,
 		double, double, double);
     void line(const img_point *, const img_point *, unsigned, bool);
-    void label(const img_point *, const char *, bool, int);
+    void label(const img_point *, const wxString&, bool, int);
     void footer();
 };
