@@ -18,9 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include "pos.h"
 
@@ -32,7 +30,6 @@
 
 #include "message.h"
 #include "namecompare.h"
-#include "osalloc.h"
 #include "useful.h"
 
 using namespace std;
